;; RADICAL EMERGENCE MODEL

extensions [profiler]

breed [techs tech] ;; instead of turtles, let's call them technologies
breed [developers developer]

techs-own [
 product ;; the "product" of the technology
 cost ;; the cost of the technology
 fitness ;; fitness value
 distance-to-nearest-peak;; and a single variable with the minimum distance to a target
 nearest-peak;; allows the techs to be divided by the nearest peak(for competition and fitness determination)
 components ;; a list of components used
 operators ;; a list of operators used
 cost-of-components ;; cost of components used 
 contributor-tech-whos ;; a list of Whos - which unique techs are used by this tech
 who-uses-me ;; a list of Whos of techs that use this technology
 how-many-use-me ;; basically a length of who-uses-me list; put here to speed up the updates!
 created ;; timestamp when developed
 last-updated ;; timestamp when updated last
 active-repertoire? ;; is tech in active use?
 number-of-consecutive-turns-in-AR ;; how many consecutive turns this tech has been in AR
 parent ;; which tech this is based from
 children-techs ;; who are my kids
 no-of-my-children ;; how many kids I have
 invented-by ;; who invented me?
 updated-this-turn? ;; for fitness calculations etc.
 debug ;; general debug variable
]

developers-own [
  strategy ;; explorer or improver
]

globals [
 list-of-operators
 list-of-peak-values ;; lists all the peak product values, or the peaks in the fitness landscape
 attempts-at-peak;; lists uncertainties associated with same position target; updated when tech variable nearest-peakincreases
 list-of-reached-peaks ;; list of peaks reached
 list-of-almost-reached-peaks ;; list of peaks techs are close by
 list-of-certain-peaks ;; list of peaks with uncertainty = 0
 list-of-valleys ;; list of "valley" values, ie. infeasible technologies
 list-of-maxed-techs ;; list of technologies with product = cost
 list-of-primitives ;; list of "primitive" technologies
 list-of-replacements ;; a list where number of replacements is saved during update, to draw a histogram
 highest-tech-reached ;; the highest product reached so far 
 peaks-reached-this-turn?;; TRUE/FALSE, are there peaks created this turn?
 list-of-products-created-this-turn ;; list of all products created this turn
 list-of-new-nearest-peaks-this-turn ;; list of "nearest peaks" of new technologies this turn; used for updating fitness
 stop-called? ;; TRUE/FALSE, mostly for debugging
 old-product ;; debugging variable...
 inventor ;; ID of developer-agent who developed this technology
 dump_ticks_internal ;; internal variable string for data dumps
 tech_data_filename ;; for setting dump file
 globals_data_filename ;; for global variables
 
 
]

to setup
  clear-all
  no-display
  let seed-number non-random_seed
  if random-seed? = TRUE [set seed-number random 100001]
  random-seed seed-number
  print(word " ")
  print(word " ")
  print(word " ")
  print(word"Random seed: " seed-number)
  print(word " ")
  
  set-default-shape techs "circle"
  
  set list-of-peak-values [] ;; setting up an empty list
  set attempts-at-peak[]
  set list-of-reached-peaks []
  set list-of-almost-reached-peaks []
  set list-of-certain-peaks []
  set list-of-valleys []
  set list-of-maxed-techs []
  
  set list-of-operators [-1 1]
  
  ;; creating peaks and valleys
 
  let max-random-tech-level max-integer - 1
 
  while [length list-of-peak-values <= ((max-integer * (density-of-peaks / 100)) - 1)] [ ;; -1 because it starts from 0
    set list-of-peak-values lput ((random max-random-tech-level) + 2) list-of-peak-values ;; +2 to prevent peaks being 0 or 1
    set list-of-peak-values remove-duplicates list-of-peak-values ;; avoiding duplicates
  ]
  set list-of-peak-values sort list-of-peak-values ;; ... and sorting
 
  ;; setting up empty list for attemps made at a target
  
  set attempts-at-peak map [0 * ?] list-of-peak-values ;; sets up zeros only
  
  
 ;; Creating valleys
  
  if density-of-peaks + density-of-valleys >= 100 [set density-of-valleys 100 - density-of-peaks] ;; Making sure that the number of valleys is not higher than max tech level
  
  if density-of-peaks < 100 [ ;; no sense trying to create valleys if everything's a target!
    let number-of-valleys round (max-integer * (density-of-valleys / 100))
    
    ;; previous version tried to randomly "fill in" the list of valleys
    ;; this version selects randomly from what's left
    ;; to avoid spending infinite time chasing that last number in situations where density of peaks + density of valleys -> 100
    let potential-valleys []
    let number 1
    while [length potential-valleys <= max-integer] [ ; create a list from 1 to max-integer
      set potential-valleys lput number potential-valleys
      set number number + 1
    ]

    set potential-valleys remove list-of-peak-values potential-valleys ;; then remove peaks from the list
    set list-of-valleys n-of number-of-valleys potential-valleys ;; then randomly pick the valleys
    set list-of-valleys sort list-of-valleys
  ]
  
  set list-of-almost-reached-peaks lput first list-of-peak-values list-of-almost-reached-peaks
  
  ;; CREATING A SET OF PRIMITIVES
  ;;
  create-techs 1 [
    set product 1  
    set cost 1
    set components [1] ;; sets up a list with 1 component
    set operators [1]
    set cost-of-components [1] ;; 
    set contributor-tech-whos [0] ;; 
    set fitness 1
    set nearest-peak first list-of-peak-values
    repeat (max-components - 1) [
      set components lput 0 components
      set cost-of-components lput 0 cost-of-components
      set contributor-tech-whos lput "n/a" contributor-tech-whos
      set operators lput 0 operators
    ]
    set who-uses-me []
    set how-many-use-me 0 
    set distance-to-nearest-peak nearest-peak - product
    set created 0 ;; primitives are gen 0
    set children-techs []
    set no-of-my-children 0
    set active-repertoire? TRUE
    set number-of-consecutive-turns-in-AR 1
    set list-of-primitives [product] of techs ;; this is needed in setting peaks, to avoid primitives being peaks
    set updated-this-turn? FALSE
  ]
  
  ;; creating developer agents
  create-developers 1 [
    set strategy "innovator"
  ]
  
  
  ;; DATA DUMP SETUP
  
  set dump_ticks_internal read-from-string dump_ticks ;; so that we get a list
  
  ;; saved with timestamp
  let hours substring date-and-time 0 2
  let mins substring date-and-time 3 5
  let seconds substring date-and-time 6 8
  let date substring date-and-time 16 27
  
  set tech_data_filename (word "ExtraInnovdT" density-of-peaks "_dV" density-of-valleys "_maxC" max-components "_" date "_" hours mins seconds "_techdata.csv")
  let settings_dump_filename (word "ExtraInnovdT" density-of-peaks "_dV" density-of-valleys "_maxC" max-components "_" date "_" hours mins seconds "_settings.csv")
  set globals_data_filename (word "ExtraInnovdT" density-of-peaks "_dV" density-of-valleys "_maxC" max-components "_" date "_" hours mins seconds "_globalsdata.csv")
  ;; so that we get 3 files, one for settings, one for data, and one for globals easier for R and other programs esp. when there are thousands of valleys and peaks
  
  
  print settings_dump_filename
  print tech_data_filename
  print globals_data_filename
  
  if data_dump? = TRUE [
    ;; let's dump the settings first
    file-open settings_dump_filename
    file-print (word "Random seed: " seed-number)
    file-print (word "Max tech level: " max-integer)
    file-print (word "Fitness function: " fitness-function)
    file-print("peaks,valleys")
    let temporary-list-of-peaks list-of-peak-values ;; we'll create temporary variables to help printing
    let temporary-list-of-valleys list-of-valleys
    while [not empty? temporary-list-of-peaks OR not empty? temporary-list-of-valleys] [
      let next-peak-value 0
      let next-valley-value 0
      ifelse not empty? temporary-list-of-peaks [set next-peak-value item 0 temporary-list-of-peaks] [set next-peak-value "NA"]
      ifelse not empty? temporary-list-of-valleys [set next-valley-value item 0 temporary-list-of-valleys] [set next-valley-value "NA"]
      file-print(word next-peak-value "," next-valley-value)
      if not empty? temporary-list-of-peaks [set temporary-list-of-peaks remove-item 0 temporary-list-of-peaks]
      if not empty? temporary-list-of-valleys [set temporary-list-of-valleys remove-item 0 temporary-list-of-valleys]
    ]
    
    file-close
    
    ;; then create a file for tech data dump
    file-open tech_data_filename
    file-print("who,parent,product,created,cost,contributor-tech-whos,operators,nearest-tgt,fitness,who-uses-me,AR?,consc-turns-in-AR,tick") ;; creates the labels at line 1, remember to edit if you change the content of the dump!
    file-close
    
    ;; and for globals
    
    file-open globals_data_filename
    file-print("list-of-reached-peaks,list-of-almost-reached-peaks,list-of-maxed-techs,highest-tech-reached,peaks-reached-this-turn?,tick")
    file-close
  ]
   

 
  reset-ticks
  
  ;; First turn - creates first 10 techs 
  ;; (the procedure is a bit different from the normal tech creation - using IF statements would be messier!
  
  
 ; while [count techs <= 11] [ ;; creates new techs until there are 10 starting techs + tech 0
 ;       create-techs 1 [
 ;         let newly-created-tech [who] of self
 ;         if debugging-mode? = TRUE [print (word "1st turn tech: " newly-created-tech)]
 ;         let replacement-position random max-components
 ;         set components [components] of tech 0
 ;         set contributor-tech-whos [contributor-tech-whos] of tech 0
 ;         set cost-of-components [cost-of-components] of tech 0
 ;         print (word"Replacement position: " replacement-position)
 ;         set operators replace-item replacement-position [operators] of tech 0 (one-of list-of-operators)
 ;         set parent [who] of tech 0
 ;         if debugging-mode? = TRUE [print (word "Parent: tech 0 Replacement: 1 Pos: " replacement-position " New comp.: 1")]
 ;         set components replace-item replacement-position components 1
 ;         set contributor-tech-whos replace-item replacement-position contributor-tech-whos [who] of tech 0
 ;         set cost-of-components replace-item replacement-position cost-of-components 1
            
          ;; determining the cost  
 ;         let operator-pos 0 ;; starts from the first operator
 ;         foreach operators [
 ;         set product product + ((item operator-pos components) * ?)
 ;         set operator-pos operator-pos + 1
 ;         ]
         
 ;         if member? product list-of-valleys [die] ;; kills off valley techs
 ;         if product < 0 [die] ;; kills off negative techs
 ;         
 ;         set created ticks
 ;         set last-updated "n/a"
 ;         set invented-by inventor
 ;         set cost sum cost-of-components
 ;         set children-techs []
 ;        set no-of-my-children 0
 ;        set who-uses-me []
 ;         set how-many-use-me 0
 ;         set number-of-consecutive-turns-in-AR 0
 ;         set updated-this-turn? FALSE
 ;         
          ;; DETERMINE DISTANCE TO peaks
          ;;
;         let distance-to-peak-product-values ( map [( ? - product  ) ] list-of-peak-values )
;         let distance-to-peak-absolute-values map abs distance-to-peak-product-values ;; this complication is needed so that we can extract the minimum absolute value
;         let position-of-distance-value position min distance-to-peak-absolute-values distance-to-peak-absolute-values ;; ... first we note the position of the lowest value (if tied, the first is picked?)
;         set distance-to-nearest-peak item position-of-distance-value distance-to-peak-product-values ;; then use the position to pick the proper value
;         set nearest-peak distance-to-nearest-peak + product
          
          ;; SET LIST OF ALMOST REACHED peaks
          ;;
;          if not member? nearest-peak list-of-almost-reached-peaks and not member? nearest-peak list-of-reached-peaks [ ;; puts the nearest peak to the list of almost reached peaks
;            set list-of-almost-reached-peaks lput nearest-peak list-of-almost-reached-peaks
;          ]
          ;; debug printing
;          if debugging-mode? = TRUE [if distance-to-nearest-peak = 0 and member? product list-of-reached-peaks [print (word self " also reached peak" product)]]
    
          ;; SET LIST OF REACHED peaks
          ;;
;          if distance-to-nearest-peak = 0 and not member? product list-of-reached-peaks [
;            set list-of-reached-peaks lput product list-of-reached-peaks
;            set list-of-almost-reached-peaks remove product list-of-almost-reached-peaks 
;            set peaks-reached-this-turn? TRUE
;            if debugging-mode? = TRUE [print (word self " is the first to reach peak" product)]
;          ]
             
          
          ;; update parent and contributor info
          
;          ask tech 0 [
;            set children-techs lput newly-created-tech children-techs
;            set no-of-my-children no-of-my-children + 1
;          ]
          
 ;         ask tech 0 [
  ;          set who-uses-me lput newly-created-tech who-uses-me
   ;         set how-many-use-me how-many-use-me + 1
;          ]
;        ]
;      ]
  
  
  tick
  
  ask techs [set-fitness] ;; sets fitness for all techs
  
end

to go
  print (word " " )
  if data_dump? = TRUE and not empty? dump_ticks_internal and ticks = item 0 dump_ticks_internal [
    dump-data
    set dump_ticks_internal remove-item 0 dump_ticks_internal
  ]
  if ticks >= stop-at-tick [stop]
  tick
  print(word"--- TICK " ticks " START ---")
  ask techs [set updated-this-turn? FALSE]
  
  ;; Ask developers to create new technologies from techs existing at the start of the turn
  print(word"--- INNOVATORS CREATE TECH ---")
  ask developers with [strategy = "innovator"] [draw-3-create-new-tech]
  ;; print(word"--- COST LEADERS IMPROVE ---")
  ;; ask developers with [strategy = "cost_leader"] [improve-existing-tech]
  
  ;; update active repertoire
 
  update-active-repertoire-fully-recursive
  update-fitnesses
end

to update-active-repertoire-fully-recursive
  ask techs [set active-repertoire? FALSE]
  let peaks-list list-of-reached-peaks
  set peaks-list sentence peaks-list list-of-almost-reached-peaks 
  let active-components []
  if debugging-mode? = TRUE [print (word "AR update - peaks are " peaks-list)]
  foreach peaks-list [
    if debugging-mode? = TRUE [print (word "AR update, target: " ?)]
    let techs-with-this-peak techs with [nearest-peak = ?]
    let max-number competition-level
    if length sort-on [(- fitness)] techs-with-this-peak < competition-level [
      set max-number length sort-on [(- fitness)] techs-with-this-peak
    ]
    let top-techs max-n-of max-number techs-with-this-peak[fitness]
    if debugging-mode? = TRUE [print (word "Top " max-number " techs: " (sort [who] of top-techs))]
    ask top-techs [
      if not member? product list-of-valleys [
        set active-repertoire? TRUE
        set active-components sentence contributor-tech-whos active-components
      ]
    ]
  ]
  
  if count techs with [active-repertoire? = TRUE] = 0 [ ;; this is to ensure AR is never empty
    let potentials techs with [not member? product list-of-valleys]
    ask max-one-of potentials [product] [
      set active-repertoire? TRUE
      set active-components sentence contributor-tech-whos active-components
    ]
  ]
      
  
  set active-components remove-duplicates active-components
  let component-to-remove 0
  let components-to-add []
  foreach active-components [
    if ? != "n/a" [
      ask tech ? [
        if active-repertoire? = FALSE [
          set active-repertoire? TRUE
          set updated-this-turn? TRUE
          set component-to-remove [who] of self
          set components-to-add contributor-tech-whos
          ;; set active-components remove [who] of self active-components
          ;; set active-components sentence contributor-tech-whos active-components ;; fully recursive
        ]
        set active-components remove component-to-remove active-components
        set active-components sentence components-to-add active-components
      ]
    ]
  ]
 ask techs with [active-repertoire? = TRUE] [set number-of-consecutive-turns-in-AR number-of-consecutive-turns-in-AR + 1]
  ;; ask techs with [active-repertoire? = FALSE] [set number-of-consecutive-turns-in-AR 0]

end

to update-fitnesses
  
  let peaks-list list-of-reached-peaks
  set peaks-list sentence list-of-almost-reached-peaks peaks-list
  print(word "peaks: " peaks-list)
  
  foreach peaks-list [
    let techs-with-this-peak techs with [updated-this-turn? = TRUE and nearest-peak = ?]
    if debugging-mode? = TRUE [print (word "Techs with peak" ? ": " [who] of techs-with-this-peak )]
    ask techs-with-this-peak[set-fitness]
  ]
end

to set-fitness
  if fitness-function = "1/|Distance to nearest target|+1" [
    set fitness (1 / (1 + abs [distance-to-nearest-peak] of self))]
  if fitness-function = "1/|Distance|+1 + target/cost" [
    set fitness (1 / (1 + abs [distance-to-nearest-peak] of self)) + ([nearest-peak] of self / [cost] of self)]
  if fitness-function = "1/|Distance|+1 * product/cost" [
    set fitness (1 / (1 + abs [distance-to-nearest-peak] of self)) * ([product] of self / [cost] of self)]
  if fitness-function = "1/|Distance|+1 + product/cost" [
    set fitness (1 / (1 + abs [distance-to-nearest-peak] of self)) + ([product] of self / [cost] of self)]
end


to draw-3-create-new-tech
  ;; save own [who] for INVENTOR variable
  set inventor [who] of self
  print (word "Developer-agent: " inventor)
  ;; select target
  
  let peaks-list list-of-reached-peaks
  set peaks-list sentence list-of-almost-reached-peaks peaks-list
  
  foreach peaks-list [
    let techs-with-this-peak techs with [nearest-peak = ?]  
    let max-number 3
    if length sort-on [( - fitness)] techs-with-this-peak < 3 [
      set max-number length sort-on [( - fitness)] techs-with-this-peak
    ]
        
    let top-techs max-n-of max-number techs-with-this-peak[fitness]
    if debugging-mode? = TRUE [print (word "Top " max-number ", unsorted: " top-techs)]
    set top-techs sort-on [( - fitness)] top-techs ;; sorts them in descending order
    if debugging-mode? = TRUE [print (word "Top " max-number ", sorted: " top-techs)]
    let rank 0
    foreach top-techs [
      let no-of-offspring item rank [10 5 3] ;; this regulates how many offspring the tech gets
      if debugging-mode? = TRUE [print (word "Parent: " ? " N:o of offspring: " no-of-offspring)]
      let parent-tech ?
      repeat no-of-offspring [
        hatch-techs 1 [
          let newly-created-tech [who] of self
          if debugging-mode? = TRUE [print (word "New tech: " newly-created-tech)]
          let random-replacement one-of techs with [active-repertoire? = TRUE]
          let replacement-position random max-components
          set components [components] of ?
          set operators [operators] of ?
          set contributor-tech-whos [contributor-tech-whos] of ?
          set cost-of-components [cost-of-components] of ?
          set parent [who] of ?
          if debugging-mode? = TRUE [print (word "Parent: " ? " Replacement: " random-replacement " Pos: " replacement-position " New comp.: " [product] of random-replacement)]
          set components replace-item replacement-position components [product] of random-replacement
          set contributor-tech-whos replace-item replacement-position contributor-tech-whos [who] of random-replacement
      ;    let replacement-position2 random max-components
      ;    set components replace-item replacement-position2 components [product] of random-replacement
      ;    set contributor-tech-whos replace-item replacement-position2 contributor-tech-whos [who] of random-replacement
          set cost-of-components replace-item replacement-position cost-of-components [cost] of random-replacement
          set operators replace-item replacement-position operators one-of list-of-operators
          
          ;; determining the product  
          let operator-pos 0 ;; starts from the first operator
          foreach operators [
          set product product + ((item operator-pos components) * ?)
          set operator-pos operator-pos + 1
          ]
          
          if member? product list-of-valleys [die] ;; kills off valley techs
 ;         if product < 0 [die] ;; kills off negative techs
          
          set created ticks
          set last-updated "n/a"
          set invented-by inventor
          set cost sum cost-of-components
          set children-techs []
          set no-of-my-children 0
          set who-uses-me []
          set how-many-use-me 0
          set number-of-consecutive-turns-in-AR 0
          set updated-this-turn? FALSE
          
          ;; DETERMINE DISTANCE TO peaks
          ;;
          let distance-to-peak-product-values ( map [( ? - product  ) ] list-of-peak-values )
          let distance-to-peak-absolute-values map abs distance-to-peak-product-values ;; this complication is needed so that we can extract the minimum absolute value
          let position-of-distance-value position min distance-to-peak-absolute-values distance-to-peak-absolute-values ;; ... first we note the position of the lowest value (if tied, the first is picked?)
          set distance-to-nearest-peak item position-of-distance-value distance-to-peak-product-values ;; then use the position to pick the proper value
          set nearest-peak distance-to-nearest-peak + product
          
          ;; SET LIST OF ALMOST REACHED peaks
          ;;
          if not member? nearest-peak list-of-almost-reached-peaks and not member? nearest-peak list-of-reached-peaks [ ;; puts the nearest peak to the list of almost reached peaks
            set list-of-almost-reached-peaks lput nearest-peak list-of-almost-reached-peaks
          ]
          ;; debug printing
          if debugging-mode? = TRUE [if distance-to-nearest-peak = 0 and member? product list-of-reached-peaks [print (word self " also reached peak" product)]]
    
          ;; SET LIST OF REACHED peaks
          ;;
          if distance-to-nearest-peak = 0 and not member? product list-of-reached-peaks [
            set list-of-reached-peaks lput product list-of-reached-peaks
            set list-of-almost-reached-peaks remove product list-of-almost-reached-peaks 
            set peaks-reached-this-turn? TRUE
            if debugging-mode? = TRUE [print (word self " is the first to reach peak" product)]
          ]
          
          set-fitness ;; calls set-fitness procedure; 
          
          ;; update parent and contributor info
          
          ask parent-tech [
            set children-techs lput newly-created-tech children-techs
            set no-of-my-children no-of-my-children + 1
          ]
          
          ask random-replacement [
            set who-uses-me lput newly-created-tech who-uses-me
            set how-many-use-me how-many-use-me + 1
          ]
        ]
      ]
      set rank rank + 1
    ]
  ]
    
    
end
    
   

to dump-data
  print(word "Data dump, tick " ticks)
  ;; TECH DATA
  ;; data to be dumped (check at Setup if you make changes):
  ;; "who,parent,product,created,cost,contributor-tech-whos,operators,nearest-tgt,fitness,who-uses-me,AR?,consc-turns-in-AR,tick"
  
  file-open tech_data_filename
  ask techs [
    file-print(word [who] of self "," [parent] of self "," [product] of self "," [created] of self "," [cost] of self "," [contributor-tech-whos] of self "," [operators] of self "," [nearest-peak] of self "," [fitness] of self "," [who-uses-me] of self "," [active-repertoire?] of self "," [number-of-consecutive-turns-in-AR] of self "," ticks)
  ]
  file-close
  
  ;; GLOBALS DATA
  ;; data to be dumped (check at Setup if you make any changes):
  ;; "list-of-reached-peaks,list-of-almost-reached-peaks,list-of-maxed-techs,list-of-replacements,highest-tech-reached,peaks-reached-this-turn?,tick"
  file-open globals_data_filename
  file-print(word list-of-reached-peaks "," list-of-almost-reached-peaks "," list-of-maxed-techs "," list-of-replacements "," highest-tech-reached "," peaks-reached-this-turn? "," ticks)
  file-close  
end 
@#$#@#$#@
GRAPHICS-WINDOW
1085
10
1330
127
0
0
86.0
1
10
1
1
1
0
1
1
1
0
0
0
0
1
1
0
ticks
30.0

BUTTON
5
80
90
113
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
5
120
90
153
Go once
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
100
160
275
193
max-components
max-components
2
50
10
1
1
NIL
HORIZONTAL

MONITOR
500
535
590
580
Top product
max [product] of techs
17
1
11

MONITOR
505
470
595
515
# of active
count techs with [active-repertoire? = TRUE]
17
1
11

BUTTON
5
160
90
193
Go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
505
425
595
470
# of techs
count techs
17
1
11

SWITCH
1000
510
1167
543
debugging-mode?
debugging-mode?
1
1
-1000

INPUTBOX
165
10
295
70
max-integer
1000
1
0
Number

SLIDER
100
80
292
113
density-of-peaks
density-of-peaks
1
100
25
1
1
%
HORIZONTAL

MONITOR
675
20
765
65
# of peaks
length list-of-peak-values
17
1
11

MONITOR
500
595
590
640
Top cost
max [cost] of techs
17
1
11

SLIDER
100
120
275
153
density-of-valleys
density-of-valleys
0
100
20
1
1
%
HORIZONTAL

MONITOR
5
265
62
310
NIL
ticks
17
1
11

PLOT
5
565
305
755
Repertoire
NIL
NIL
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Active" 1.0 0 -16777216 true "" "plot count techs with [active-repertoire? = TRUE]"

MONITOR
675
120
787
165
Reached peaks
length list-of-reached-peaks
17
1
11

MONITOR
200
265
277
310
On target
count techs with [distance-to-nearest-peak = 0]
17
1
11

PLOT
595
595
895
785
Cost
Ticks
Cost
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Max" 1.0 0 -16777216 true "" "plot max [cost] of techs"
"Mean" 1.0 0 -7500403 true "" "plot mean [cost] of techs"

MONITOR
70
265
197
310
Actives on target
count techs with [active-repertoire? = TRUE and distance-to-nearest-peak = 0]
17
1
11

MONITOR
675
70
762
115
Near peaks
length list-of-almost-reached-peaks
17
1
11

PLOT
315
10
670
230
Peaks
Ticks
Peaks
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Almost reached Peaks" 1.0 0 -7500403 true "" "plot length list-of-almost-reached-peaks"
"Peaks total" 1.0 0 -2674135 true "" "plot length list-of-peak-values"
"Reached Peaks" 1.0 0 -955883 true "" "plot length list-of-reached-peaks"

PLOT
600
390
895
585
Top product over time
Ticks
Tech level
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"Highest tech reached" 1.0 0 -16777216 true "" "plot max [product] of techs"

INPUTBOX
5
10
157
70
stop-at-tick
30
1
0
Number

PLOT
5
320
360
545
Spread of who-uses-me of techs
Users
No of techs
1.0
100.0
0.0
100.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" "histogram [how-many-use-me] of techs"

MONITOR
675
170
765
215
Valleys
length list-of-valleys
17
1
11

CHOOSER
335
265
582
310
fitness-function
fitness-function
"1/|Distance to nearest target|+1" "1/|Distance|+1 * product/cost" "1/|Distance|+1 + product/cost"
1

INPUTBOX
935
620
1165
735
dump_ticks
[30]
1
0
String

SWITCH
1030
565
1162
598
data_dump?
data_dump?
0
1
-1000

SWITCH
1015
400
1165
433
random-seed?
random-seed?
0
1
-1000

INPUTBOX
1015
440
1167
500
non-random_seed
123456
1
0
Number

SLIDER
85
220
262
253
competition-level
competition-level
1
20
3
1
1
NIL
HORIZONTAL

PLOT
805
125
1165
385
Offspring
Parents
Offspring
1.0
100.0
0.0
100.0
true
false
"" ""
PENS
"kiddos" 1.0 1 -16777216 true "" "histogram [no-of-my-children] of techs"

@#$#@#$#@
## WHAT IS IT?

ADDER models the evolution of complex technological systems from simple components. Technologies are represented by  their "products," or natural numbers (1,2,3,...,n). For example, "Tech 4" means a technology whose "product" is number 4. 

New technologies are created by combining (adding) existing technologies. For example, a Tech 7 could be composed of Techs 4 and 1 as follows: [4 + 4 - 1]. Under default settings, every new technology can be used as a component in  the future; therefore, once Tech 7 is created, a Tech 14 could be created by [7 + 7]. The rules by which technologies are selected and how many components a technology can have may vary depending on the simulation setting. 

## HOW IT WORKS

The model starts from a predefined set of "primitives" or primitive technologies, which are initialized in the Setup ("Creating a set of primitives"). Under default settings, there is one primitive, which has a product 1 and a cost of 1.

### Variables explained

#### Techs-own
* _product:_ the "product" of technology; what it produces.
* _cost:_ the cost of technology, calculated as a sum of cost of component technologies. By default, each primitive used has a cost of 1, so a tech [1 + 1 - 1] would have a product 2 and cost 3. 
* _distance-to-nearest-target:_ the distance to nearest target value. Either positive or negative.
* _nearest-peak:_ the target value that is closest to the tech's product.
* _components:_ a list of component technologies used, e.g. [4 3 1 0 0]. Zeros represent "no tech" and are there to allow for variable number of components (up to the Interface variable max-components; in the above example, max-components = 5). In the default version, this list remains unchanging from the creation of the technology. 
operators: a list of operators (+ or -) for each component. A tech's product is determined by adding components together with appropriate operators. To continue the above example, the tech might have a following list of operators: [1 1 -1 1 1]. Therefore, the tech's product would be 4 + 3 - 1 + 0 + 0 = 6. 
* _cost-of-components:_ a list for the cost of components used. This list is updated if component technologies are updated (see below).
* _contributor-tech-whos:_ a list of technologies (by [who] ID) which are being used as components in this technology. Used to establish relationship between technologies and in updating the cost of components.
* _who-uses-me:_ the same as above but in reverse; the [who]s of technologies that use this technology as a component.
* _how-many-use-me:_ The length of above list. For performance reasons, calculated for each tech only when the tech is updated (another option would be to use LENGTH reporter). 
* _tech-ID:_ somewhat redundant alternative ID for technology, may be deleted in later versions.
* _created:_ timestamp (tick) when the technology is created.
* _last-updated:_ timestamp (tick) when the technology has been last updated (ie. when its components have become cheaper).
* _active-repertoire?_ TRUE/FALSE variable, whether the technology is in active use ie. can be used as a component for future technologies.
* _debug:_ catch-all variable for debugging.

#### Globals:
* _next-tech-ID:_ a counter for next tech ID.
* _list-of-operators:_ a list of allowed operators. Set in Setup. By default, + and -.
* _list-of-target-values:_ list of "targets" that technologies try to satisfy (e.g. 223). Set up in Setup, "Creating the peaks" (and initialized in the beginning of the setup).
* _list-of-reached-targets:_ a list, originally initialized as empty, that is populated by target values that have been reached exactly. The list is updated in technology creation phase, if a viable technology that satisfies the target exactly is created.
* _list-of-almost-reached-targets:_ In the current version, a target is almost reached if a technology with a product closer to that target than any other (e.g. already reached target) is created. This list is then updated. When a target is reached, it is added to * list-of-reached-targets and removed from this list.
* _list-of-valleys:_ a list of technologies that are not viable and cannot be created.
* _list-of-techs-with-efficiency-maxed-out:_ a list of technologies that cannot be improved any further. In default model, this means technologies whose cost equals product (as the cheapest way to produce, say, 4 is with four primitives [1+1+1+1], which has a cost of 4). This list is updated at the technology creation procedure.
* _failed-techs:_ a counter that is advanced when no tech is created this turn. For statistics.
* _cumulative-replacements:_ a counter of cumulative replacements of technologies. For statistics.
* _list-of-replacements:_ For statistics, used to draw a histogram in Interface screen.
* _highest-tech-reached:_ the highest tech level (product) reached so far.
* _list-of-techlevels-found:_ a list of all products created so far.
* _size-of-active-repertoire:_ a counter for statistics, tracking the size of active repertoire.
* _viable-techs-created-this-turn?_ A TRUE/FALSE variable that is set in the tech creation phase, after a prospective tech is drawn but before actually creating it.
* _last-techlevel-created:_ the product of the last technology created.
* _last-tech-who-created:_ [who] of last tech created so far.
* _stop-called?_ TRUE/FALSE variable that can be used by procedures to call a stop to the simulation.
* _obsoleted-techs:_ a list of technologies that have become obsoleted this turn. Used in the update sequence.
* _obsolete-tech-who:_ Used in the update sequence to move the [who] of current obsoleted tech to the update procedure.
* _prospective-_ : a set of global variables used in the creation of new technologies. A new tech is first created as a "prospective" tech, and "committed" to a newly-created agent ("tech") only if it satisfies the conditions set at the beginning of the simulation (e.g. it's not a "valley" tech).

### The procedures explained

#### The Setup procedure:
1. clears all variables
2. sets a random seed
3. sets a selected visualization display (ln or decimal ie. no scaling)
4. initializes lists and variables; mostly empty lists, but this sets up the list-of-operators
5. creates a set of primitives
6. creates the targets and the valleys of the "landscape:"
 1. the number of targets depends on the density-of-targets variable and the maximum tech level (stops-at-tech-level).
 2. the number of valleys depends on the density-of-valleys variable, so that density-of-valleys + density-of-targets ≤ 100 %
7. resets ticks
8. prints timestamp and ADDER READY

#### The Go procedure:
1. checks if conditions are reached for the simulation to stop
advances the tick
2. clears variables viable-techs and obsoleted-techs to FALSE and []
updates active repertoire, according to selected update model:
    1. least-cost
    2. distance-to-target-and-cost
3. updates visualization, if necessary
4. sets statistics variable size-of-active-repertoire
5. creates new technology according to selected new tech creation model
6. calls a procedure to check whether techs need updating
7. clears who-uses-me variables from duplicates

### UPDATE SCHEMES

####update-active-repertoire-least-cost:

1. begins a foreach loop for a list of products of technologies, with duplicates removed
2. for each product, the cheapest technology is added to the active repertoire

#### update-active-repertoire-check-distance-to-target-and-cost:
1. begins a foreach loop for each of almost-reached targets
    1. for each of these, selects the tech that is closest to the target
      1. if several techs are equally close, selects the cheapest
2. begins a foreach loop for each reached target
  1. picks the technology which is exactly on the target and has the lowest cost


### TECHNOLOGY CREATION PROCEDURES

#### randomize-new-tech:
1. clears "prospective-" variables
2. picks 2 technologies from the active repertoire as first components (this is to prevent the creation of technologies with 0 or 1 components)
    1. picks random operators for these technologies
    2. saves cost data for the components
3. repeats until max-components - 2 is reached:
    1. picks a technology from active repertoire
    2. randomizes whether the component is added, or not (in which case a 0 is added)
    3. picks a random operator
    4. if a (non-zero) component is added: 
    1. saves cost data
    2. saves contributor-techs [who] to prospective-contributor-tech-whos list
  5. if no component is added (above randomizer gets a 0):
    1. saves a 0 to the list of component costs
    2. saves a "na" to the list of prospective contributor tech whos list (to retain the correct positions for costs, whos &c.)

3. determines the product: combines components with operators and adds them up

4. determines distance to target values:
    1. creates a distance-to-target-product-values list by comparing the product with target values
    2. creates a version of the above list with absolute values, so that <min> function works properly
    3. checks the position of minimum absolute value on the list
    4. uses this position to pick the proper distance to target value from the distance-to-target-product-values list
5. checks whether the created tech is "viable" (ie. allowed)
    1. checks against list of valleys
    2. checks against list of techs with efficiency maxed out (cost = product)
    3. checks whether the tech is negative or not, and whether negative techs are allowed
    4. checks the tech is an improvement to the existing situation
    5. if conditions are not satisfied, sets viable-techs-created-this-turn? and proceed-to-tech-creation? to FALSE

6. checks whether a tech can be created (proceed-to-tech-creation? = TRUE); if yes, calls the procedure create-next-tech

#### Creating next technology:
1. inherits variables from prospective-variables
3. sets last-techlevel-created as a product
4. sets up other variables
5. sets viable-techs-created-this-turn? to TRUE
6. checks whether new targets are to be added to list-of-almost-reached targets
7. checks whether new targets are to be added to the list-of-reached-targets
    1. if yes, removes the target from the list-of-almost-reached-targets
8. checks if the tech is now maxed out in efficiency (|product| = cost; absolute value to allow negative technologies if desired)
    1. if yes, adds to the list-of-techs-with-efficiency-maxed-out
9. checks if technology frontier (highest tech reached) advances
10. updates position in the visualization

### CHECKING WHETHER TO UPDATE TECHNOLOGIES
1. if viable-techs-created-this-turn? = TRUE:
    1. if there are existing technologies with the same tech level than the one created this turn:
      1. populates the list obsoleted-techs with techs that i) have the same tech level and ii) are more expensive than the last tech created
    2. begins a while loop until obsoleted-tech list is empty
      1. stores the current obsoleted-tech to a variable obsolete-tech-who
      2. calls for update-techs procedure

### UPDATING TECHNOLOGIES

#### update-techs-simple:

1. picks an agentset with the same product as the obsolete-tech-who. This set includes the obsoleted technology, as it's possible that there are no replacements but the technology has just gotten cheaper.
    1. out of these, picks the cheapest as a replacement
2. asks other techs:
    1. if the obsoleted-tech in question is used by them:
    2. determines the position in the component list
    3. replaces the [who] in contributor-tech-whos with the replacement's [who]
    4. replaces the cost in cost-of-components list with the replacement's cost
    5. recalculates cost
    6. sets last-updated variable with a timestamp
    7. removes the updated tech's [who] from the who-uses-me list of the obsoleted technology
    8. adds the updated tech's [who] to the who-uses-me list of the replacement technology
    9. updates position in the visualization
3. if someone is using this technology and the update reduces costs
    1. and if the update has reduced costs
    2. adds the updated tech's [who] to the list obsoleted-techs
4. once all the other technologies are checked, the obsoleted technology is removed from the obsoleted-techs list
5. if at least one technology has been updated, the list-of-replacements is updated with the information of how many techs were updated this turn (gives the cascade size)

## HOW TO USE IT

This section could explain how to use the model, including a description of each of the items in the interface tab.

## THINGS TO NOTICE

This section could give some ideas of things for the user to notice while running the model.

## THINGS TO TRY

This section could give some ideas of things for the user to try to do (move sliders, switches, etc.) with the model.

## EXTENDING THE MODEL

This section could give some ideas of things to add or change in the procedures tab to make the model more complicated, detailed, accurate, etc.

## NETLOGO FEATURES

This section could point out any especially interesting or unusual features of NetLogo that the model makes use of, particularly in the Procedures tab.  It might also point out places where workarounds were needed because of missing features.

## RELATED MODELS

This section could give the names of models in the NetLogo Models Library or elsewhere which are of related interest.

## CREDITS AND REFERENCES

This section could contain a reference to the model's URL on the web if it has one, as well as any other necessary credits or references.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
0
Rectangle -7500403 true true 151 225 180 285
Rectangle -7500403 true true 47 225 75 285
Rectangle -7500403 true true 15 75 210 225
Circle -7500403 true true 135 75 150
Circle -16777216 true false 165 76 116

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="FirstExp" repetitions="10" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>count turtles</metric>
    <enumeratedValueSet variable="stop-at-tick">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="random-seed?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="non-random_seed">
      <value value="123456"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-components">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-peaks">
      <value value="25"/>
      <value value="50"/>
      <value value="75"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-valleys">
      <value value="20"/>
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="debugging-mode?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-integer">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="data_dump?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="dump_ticks">
      <value value="&quot;[30]&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;1/|Distance|+1 * product/cost&quot;"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="Coarse_Parameter_Sweep1" repetitions="5" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>count turtles</metric>
    <steppedValueSet variable="density-of-valleys" first="32" step="10" last="69"/>
    <steppedValueSet variable="density-of-targets" first="1" step="10" last="30"/>
    <enumeratedValueSet variable="non-random_seed">
      <value value="123456"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="data_dump?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-tech-level">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;1/|Distance|+1 * product/cost&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="visualization?">
      <value value="&quot;None&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="debugging-mode?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-components">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="random-seed?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="dump_ticks">
      <value value="&quot;[1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50]&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="stop-at-tick">
      <value value="40"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="Coarse_Parameter_Sweep2" repetitions="5" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>count turtles</metric>
    <steppedValueSet variable="density-of-valleys" first="1" step="10" last="30"/>
    <steppedValueSet variable="density-of-targets" first="32" step="10" last="69"/>
    <enumeratedValueSet variable="non-random_seed">
      <value value="123456"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="data_dump?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-tech-level">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;1/|Distance|+1 * product/cost&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="visualization?">
      <value value="&quot;None&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="debugging-mode?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-components">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="random-seed?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="dump_ticks">
      <value value="&quot;[1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50]&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="stop-at-tick">
      <value value="40"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="Coarse_Parameter_Sweep3" repetitions="5" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>count turtles</metric>
    <enumeratedValueSet variable="debugging-mode?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="non-random_seed">
      <value value="123456"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-components">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="visualization?">
      <value value="&quot;None&quot;"/>
    </enumeratedValueSet>
    <steppedValueSet variable="density-of-targets" first="1" step="10" last="31"/>
    <enumeratedValueSet variable="max-tech-level">
      <value value="1000"/>
    </enumeratedValueSet>
    <steppedValueSet variable="density-of-valleys" first="1" step="10" last="31"/>
    <enumeratedValueSet variable="random-seed?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="dump_ticks">
      <value value="&quot;[1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50]&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="data_dump?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;1/|Distance|+1 * product/cost&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="stop-at-tick">
      <value value="20"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="SecondExp" repetitions="20" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>count turtles</metric>
    <enumeratedValueSet variable="stop-at-tick">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="random-seed?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="non-random_seed">
      <value value="123456"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-components">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-peaks">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-valleys">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="debugging-mode?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-integer">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="data_dump?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="dump_ticks">
      <value value="&quot;[30]&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;1/|Distance|+1 * product/cost&quot;"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="FirstExp" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>count turtles</metric>
    <enumeratedValueSet variable="stop-at-tick">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="random-seed?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="non-random_seed">
      <value value="123456"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-components">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-peaks">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-valleys">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="debugging-mode?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-integer">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="data_dump?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="dump_ticks">
      <value value="&quot;[30]&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;1/|Distance|+1 * product/cost&quot;"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
1
@#$#@#$#@
